@@ -18,6 +18,12 @@ control = {
     "loop": constants.LOOP,
     "looped group": constants.REPEAT,
 }
+select_from_file = {
+    "select_one_from_file": constants.SELECT_ONE,
+    "select_multiple_from_file": constants.SELECT_ALL_THAT_APPLY,
+    "select one from file": constants.SELECT_ONE,
+    "select multiple from file": constants.SELECT_ALL_THAT_APPLY,
+}
 select = {
     "add select one prompt using": constants.SELECT_ONE,
     "add select multiple prompt using": constants.SELECT_ALL_THAT_APPLY,
@@ -28,11 +34,8 @@ select = {
     "select one": constants.SELECT_ONE,
     "select_multiple": constants.SELECT_ALL_THAT_APPLY,
     "select all that apply": constants.SELECT_ALL_THAT_APPLY,
-    "select_one_external": "select one external",
-    "select_one_from_file": constants.SELECT_ONE,
-    "select_multiple_from_file": constants.SELECT_ALL_THAT_APPLY,
-    "select one from file": constants.SELECT_ONE,
-    "select multiple from file": constants.SELECT_ALL_THAT_APPLY,
+    "select_one_external": constants.SELECT_ONE_EXTERNAL,
+    **select_from_file,
     "rank": constants.RANK,
 }
 cascading = {
@@ -108,6 +111,9 @@ survey_header = {
     "parameters": "parameters",
     constants.ENTITIES_SAVETO: "bind::entities:saveto",
 }
+
+entities_header = {"list_name": "dataset"}
+
 # Key is the pyxform internal name, Value is the name used in error/warning messages.
 TRANSLATABLE_SURVEY_COLUMNS = {
     constants.LABEL: constants.LABEL,
