@@ -8,6 +8,7 @@ are easier to find.
 """
 # TODO: Replace matching strings in the json2xforms code (builder.py,
 # survey.py, survey_element.py, question.py) with these constants
+from pyxform.util.enum import StrEnum
 
 TYPE = "type"
 TITLE = "title"
@@ -47,6 +48,10 @@ BIND = (
 MEDIA = "media"
 CONTROL = "control"
 APPEARANCE = "appearance"
+ITEMSET = "itemset"
+RANDOMIZE = "randomize"
+CHOICE_FILTER = "choice_filter"
+PARAMETERS = "parameters"
 
 LOOP = "loop"
 COLUMNS = "columns"
@@ -56,8 +61,12 @@ GROUP = "group"
 CHILDREN = "children"
 
 SELECT_ONE = "select one"
+SELECT_ONE_EXTERNAL = "select one external"
 SELECT_ALL_THAT_APPLY = "select all that apply"
+SELECT_OR_OTHER_SUFFIX = " or specify other"
 RANK = "rank"
+QUESTION = "question"
+CHOICE = "choice"
 CHOICES = "choices"
 
 # XLS Specific constants
@@ -65,6 +74,7 @@ LIST_NAME = "list name"
 CASCADING_SELECT = "cascading_select"
 TABLE_LIST = "table-list"  # hyphenated because it goes in appearance, and convention for appearance column is dashes # noqa
 FIELD_LIST = "field-list"
+LIST_NOLABEL = "list-nolabel"
 
 SURVEY = "survey"
 SETTINGS = "settings"
@@ -103,10 +113,21 @@ EXTERNAL_INSTANCES = ["calculate", "constraint", "readonly", "required", "releva
 CURRENT_XFORMS_VERSION = "1.0.0"
 
 # The ODK entities spec version that generated forms comply to
-CURRENT_ENTITIES_VERSION = "2022.1.0"
-ENTITY_RELATED = "entity_related"
+ENTITIES_CREATE_VERSION = "2022.1.0"
+CURRENT_ENTITIES_VERSION = "2023.1.0"
+ENTITY = "entity"
+ENTITY_FEATURES = "entity_features"
 ENTITIES_RESERVED_PREFIX = "__"
 
+
+class EntityColumns(StrEnum):
+    DATASET = "dataset"
+    ENTITY_ID = "entity_id"
+    CREATE_IF = "create_if"
+    UPDATE_IF = "update_if"
+    LABEL = "label"
+
+
 DEPRECATED_DEVICE_ID_METADATA_FIELDS = ["subscriberid", "simserial"]
 
 AUDIO_QUALITY_VOICE_ONLY = "voice-only"
