@@ -10,14 +10,15 @@ import xml.etree.ElementTree as ETree
 from collections import defaultdict
 from datetime import datetime
 from functools import lru_cache
+from typing import Generator, Iterator, List, Optional, Tuple
 
-from pyxform import constants
+from pyxform import aliases, constants
 from pyxform.constants import EXTERNAL_INSTANCE_EXTENSIONS
 from pyxform.errors import PyXFormError, ValidationError
 from pyxform.external_instance import ExternalInstance
 from pyxform.instance import SurveyInstance
-from pyxform.instance_info import InstanceInfo
-from pyxform.question import Question
+from pyxform.parsing import instance_expression
+from pyxform.question import Option, Question
 from pyxform.section import Section
 from pyxform.survey_element import SurveyElement
 from pyxform.utils import (
@@ -25,6 +26,7 @@ from pyxform.utils import (
     LAST_SAVED_INSTANCE_NAME,
     LAST_SAVED_REGEX,
     NSMAP,
+    DetachableElement,
     PatchedText,
     get_languages_with_bad_tags,
     has_dynamic_label,
@@ -32,7 +34,36 @@ from pyxform.utils import (
 )
 from pyxform.validators import enketo_validate, odk_validate
 
+RE_BRACKET = re.compile(r"\[([^]]+)\]")
+RE_FUNCTION_ARGS = re.compile(r"\b[^()]+\((.*)\)$")
+RE_INDEXED_REPEAT = re.compile(r"indexed-repeat\([^)]+\)")
+RE_INSTANCE = re.compile(r"instance\([^)]+.+")
+RE_INSTANCE_SECONDARY_REF = re.compile(
+    r"(instance\(.*\)\/root\/item\[.*?(\$\{.*\})\]\/.*?)\s"
+)
 RE_PULLDATA = re.compile(r"(pulldata\s*\(\s*)(.*?),")
+RE_XML_OUTPUT = re.compile(r"\n.*(<output.*>)\n(\s\s)*")
+RE_XML_TEXT = re.compile(r"(>)\n\s*(\s[^<>\s].*?)\n\s*(\s</)", re.DOTALL)
+
+
+class InstanceInfo:
+    """Standardise Instance details relevant during XML generation."""
+
+    __slots__ = ("type", "context", "name", "src", "instance")
+
+    def __init__(
+        self,
+        type: str,
+        context: Optional[str],
+        name: str,
+        src: Optional[str],
+        instance: "DetachableElement",
+    ):
+        self.type: str = type
+        self.context: Optional[str] = context
+        self.name: str = name
+        self.src: Optional[str] = src
+        self.instance: "DetachableElement" = instance
 
 
 def register_nsmap():
@@ -45,7 +76,7 @@ def register_nsmap():
 register_nsmap()
 
 
-@lru_cache(maxsize=None)
+@lru_cache(maxsize=65536)  # 2^16
 def is_parent_a_repeat(survey, xpath):
     """
     Returns the XPATH of the first repeat of the given xpath in the survey,
@@ -61,7 +92,7 @@ def is_parent_a_repeat(survey, xpath):
     return is_parent_a_repeat(survey, parent_xpath)
 
 
-@lru_cache(maxsize=None)
+@lru_cache(maxsize=65536)  # 2^16
 def share_same_repeat_parent(survey, xpath, context_xpath, reference_parent=False):
     """
     Returns a tuple of the number of steps from the context xpath to the shared
@@ -139,6 +170,19 @@ def share_same_repeat_parent(survey, xpath, context_xpath, reference_parent=Fals
     return (None, None)
 
 
+@lru_cache(maxsize=65536)  # 2^16
+def is_label_dynamic(label: str) -> bool:
+    return (
+        label is not None
+        and isinstance(label, str)
+        and re.search(BRACKETED_TAG_REGEX, label) is not None
+    )
+
+
+def recursive_dict():
+    return defaultdict(recursive_dict)
+
+
 class Survey(Section):
     """
     Survey class - represents the full XForm XML.
@@ -162,7 +206,7 @@ class Survey(Section):
             constants.COMPACT_DELIMITER: str,
             "file_name": str,
             "default_language": str,
-            "_translations": dict,
+            "_translations": recursive_dict,
             "submission_url": str,
             "auto_send": str,
             "auto_delete": str,
@@ -173,7 +217,7 @@ class Survey(Section):
             "style": str,
             "attribute": dict,
             "namespaces": str,
-            constants.ENTITY_RELATED: str,
+            constants.ENTITY_FEATURES: list,
         }
     )  # yapf: disable
 
@@ -206,7 +250,7 @@ class Survey(Section):
     def get_nsmap(self):
         """Add additional namespaces"""
         namespaces = getattr(self, constants.NAMESPACES, "")
-        if getattr(self, constants.ENTITY_RELATED, "false") == "true":
+        if len(getattr(self, constants.ENTITY_FEATURES, [])) > 0:
             namespaces += " entities=http://www.opendatakit.org/xforms/entities"
 
         if namespaces and isinstance(namespaces, str):
@@ -238,7 +282,7 @@ class Survey(Section):
         self._setup_xpath_dictionary()
 
         for triggering_reference in self.setvalues_by_triggering_ref.keys():
-            if not (re.match(BRACKETED_TAG_REGEX, triggering_reference)):
+            if not re.search(BRACKETED_TAG_REGEX, triggering_reference):
                 raise PyXFormError(
                     "Only references to other fields are allowed in the 'trigger' column."
                 )
@@ -255,46 +299,45 @@ class Survey(Section):
             "h:html",
             node("h:head", node("h:title", self.title), self.xml_model()),
             node("h:body", *self.xml_control(), **body_kwargs),
-            **nsmap
+            **nsmap,
         )
 
     def get_setvalues_for_question_name(self, question_name):
         return self.setvalues_by_triggering_ref.get("${%s}" % question_name)
 
-    @staticmethod
-    def _generate_static_instances(list_name, choice_list):
+    def _generate_static_instances(self, list_name, choice_list) -> InstanceInfo:
         """
-        Generates <instance> elements for static data
-        (e.g. choices for select type questions)
-
-        Note that per commit message 0578242 and in xls2json.py R539, an
-        instance is only output for select items defined in the choices sheet
-        when the item has a choice_filter, and it is that way for backwards
-        compatibility.
+        Generate <instance> elements for static data (e.g. choices for selects)
         """
         instance_element_list = []
-        multi_language = isinstance(choice_list[0].get("label"), dict)
         has_media = bool(choice_list[0].get("media"))
+        has_dyn_label = has_dynamic_label(choice_list)
+        multi_language = False
+        if isinstance(self._translations, dict):
+            choices = tuple(
+                k
+                for items in self._translations.values()
+                for k, v in items.items()
+                if v.get(constants.TYPE, "") == constants.CHOICE
+                and k.split("-")[0] == list_name
+            )
+            if 0 < len(choices):
+                multi_language = True
 
         for idx, choice in enumerate(choice_list):
             choice_element_list = []
-            # Add a unique id to the choice element in case there is itext
-            # it references
-            if (
-                multi_language
-                or has_media
-                or has_dynamic_label(choice_list, multi_language)
-            ):
+            # Add a unique id to the choice element in case there are itext references
+            if multi_language or has_media or has_dyn_label:
                 itext_id = "-".join([list_name, str(idx)])
                 choice_element_list.append(node("itextId", itext_id))
 
-            for name, value in sorted(choice.items()):
+            for name, value in choice.items():
                 if isinstance(value, str) and name != "label":
                     choice_element_list.append(node(name, str(value)))
                 if (
                     not multi_language
                     and not has_media
-                    and not has_dynamic_label(choice_list, multi_language)
+                    and not has_dyn_label
                     and isinstance(value, str)
                     and name == "label"
                 ):
@@ -311,7 +354,7 @@ class Survey(Section):
         )
 
     @staticmethod
-    def _generate_external_instances(element):
+    def _generate_external_instances(element) -> Optional[InstanceInfo]:
         if isinstance(element, ExternalInstance):
             name = element["name"]
             extension = element["type"].split("-")[0]
@@ -330,7 +373,7 @@ class Survey(Section):
         return None
 
     @staticmethod
-    def _validate_external_instances(instances):
+    def _validate_external_instances(instances) -> None:
         """
         Must have unique names.
 
@@ -359,7 +402,7 @@ class Survey(Section):
             raise ValidationError("\n".join(errors))
 
     @staticmethod
-    def _generate_pulldata_instances(element):
+    def _generate_pulldata_instances(element) -> Optional[List[InstanceInfo]]:
         def get_pulldata_functions(element):
             """
             Returns a list of different pulldata(... function strings if
@@ -383,9 +426,9 @@ class Survey(Section):
             uri = "jr://file-csv/{}.csv".format(file_id)
 
             return InstanceInfo(
-                type=u"pulldata",
+                type="pulldata",
                 context="[type: {t}, name: {n}]".format(
-                    t=element[u"parent"][u"type"], n=element[u"parent"][u"name"]
+                    t=element["parent"]["type"], n=element["parent"]["name"]
                 ),
                 name=file_id,
                 src=uri,
@@ -409,7 +452,7 @@ class Survey(Section):
         return None
 
     @staticmethod
-    def _generate_from_file_instances(element):
+    def _generate_from_file_instances(element) -> Optional[InstanceInfo]:
         itemset = element.get("itemset")
         file_id, ext = os.path.splitext(itemset)
         if itemset and ext in EXTERNAL_INSTANCE_EXTENSIONS:
@@ -429,9 +472,11 @@ class Survey(Section):
 
         return None
 
-    # True if a last-saved instance should be generated, false otherwise
     @staticmethod
-    def _generate_last_saved_instance(element):
+    def _generate_last_saved_instance(element) -> bool:
+        """
+        True if a last-saved instance should be generated, false otherwise.
+        """
         for expression_type in constants.EXTERNAL_INSTANCES:
             last_saved_expression = re.search(
                 LAST_SAVED_REGEX, str(element["bind"].get(expression_type))
@@ -439,12 +484,13 @@ class Survey(Section):
             if last_saved_expression:
                 return True
 
-        return re.search(LAST_SAVED_REGEX, str(element["choice_filter"])) or re.search(
-            LAST_SAVED_REGEX, str(element["default"])
+        return bool(
+            re.search(LAST_SAVED_REGEX, str(element["choice_filter"]))
+            or re.search(LAST_SAVED_REGEX, str(element["default"]))
         )
 
     @staticmethod
-    def _get_last_saved_instance():
+    def _get_last_saved_instance() -> InstanceInfo:
         name = "__last-saved"  # double underscore used to minimize risk of name conflicts
         uri = "jr://instance/last-saved"
 
@@ -456,7 +502,7 @@ class Survey(Section):
             instance=node("instance", id=name, src=uri),
         )
 
-    def _generate_instances(self):
+    def _generate_instances(self) -> Iterator[DetachableElement]:
         """
         Get instances from all the different ways that they may be generated.
 
@@ -553,8 +599,17 @@ class Survey(Section):
         self._add_empty_translations()
 
         model_kwargs = {"odk:xforms-version": constants.CURRENT_XFORMS_VERSION}
-        if getattr(self, constants.ENTITY_RELATED, "false") == "true":
-            model_kwargs["entities:entities-version"] = constants.CURRENT_ENTITIES_VERSION
+
+        entity_features = getattr(self, constants.ENTITY_FEATURES, [])
+        if len(entity_features) > 0:
+            if "update" in entity_features:
+                model_kwargs[
+                    "entities:entities-version"
+                ] = constants.CURRENT_ENTITIES_VERSION
+            else:
+                model_kwargs[
+                    "entities:entities-version"
+                ] = constants.ENTITIES_CREATE_VERSION
 
         model_children = []
         if self._translations:
@@ -606,7 +661,11 @@ class Survey(Section):
 
     def _add_to_nested_dict(self, dicty, path, value):
         if len(path) == 1:
-            dicty[path[0]] = value
+            key = path[0]
+            if key in dicty and isinstance(dicty[key], dict) and isinstance(value, dict):
+                dicty[key].update(value)
+            else:
+                dicty[key] = value
             return
         if path[0] not in dicty:
             dicty[path[0]] = {}
@@ -618,36 +677,73 @@ class Survey(Section):
         setup media and itext functions
         """
 
-        def _setup_choice_translations(name, choice_value, itext_id):
-            for media_type_or_language, value in choice_value.items():  # noqa
+        def _setup_choice_translations(
+            name, choice_value, itext_id
+        ) -> Generator[Tuple[List[str], str], None, None]:
+            for media_or_lang, value in choice_value.items():  # noqa
                 if isinstance(value, dict):
                     for language, val in value.items():
-                        self._add_to_nested_dict(
-                            self._translations,
-                            [language, itext_id, media_type_or_language],
-                            val,
-                        )
+                        yield ([language, itext_id, media_or_lang], val)
                 else:
-                    if name == "media":
-                        self._add_to_nested_dict(
-                            self._translations,
-                            [self.default_language, itext_id, media_type_or_language],
-                            value,
-                        )
+                    if name == constants.MEDIA:
+                        yield ([self.default_language, itext_id, media_or_lang], value)
                     else:
-                        self._add_to_nested_dict(
-                            self._translations,
-                            [media_type_or_language, itext_id, "long"],
-                            value,
+                        yield ([media_or_lang, itext_id, "long"], value)
+
+        itemsets_multi_language = set()
+        itemsets_has_media = set()
+        itemsets_has_dyn_label = set()
+
+        def get_choices():
+            for list_name, choice_list in self.choices.items():
+                multi_language = False
+                has_media = False
+                dyn_label = False
+                choices = []
+                for idx, choice in enumerate(choice_list):
+                    for col_name, choice_value in choice.items():
+                        lang_choice = None
+                        if col_name == constants.MEDIA:
+                            has_media = True
+                        if isinstance(choice_value, dict):
+                            lang_choice = choice_value
+                            multi_language = True
+                        elif col_name == constants.LABEL:
+                            lang_choice = {self.default_language: choice_value}
+                            if is_label_dynamic(choice_value):
+                                dyn_label = True
+                        if lang_choice is not None:
+                            # e.g. (label, {"default": "Yes"}, "consent", 0)
+                            choices.append((col_name, lang_choice, list_name, idx))
+                if multi_language or has_media or dyn_label:
+                    if multi_language:
+                        itemsets_multi_language.add(list_name)
+                    if has_media:
+                        itemsets_has_media.add(list_name)
+                    if dyn_label:
+                        itemsets_has_dyn_label.add(list_name)
+                    for c in choices:
+                        yield from _setup_choice_translations(
+                            c[0], c[1], f"{c[2]}-{c[3]}"
                         )
 
-        self._translations = defaultdict(dict)  # pylint: disable=W0201
+        for path, value in get_choices():
+            last_path = path.pop()
+            leaf_value = {last_path: value, constants.TYPE: constants.CHOICE}
+            self._add_to_nested_dict(self._translations, path, leaf_value)
+
+        select_types = set(aliases.select.keys())
         for element in self.iter_descendants():
-            # Skip creation of translations for choices in filtered selects
-            # The creation of these translations is done futher below in this
-            # function
+            itemset = element.get("itemset")
+            if itemset is not None:
+                element._itemset_multi_language = itemset in itemsets_multi_language
+                element._itemset_has_media = itemset in itemsets_has_media
+                element._itemset_dyn_label = itemset in itemsets_has_dyn_label
+
+            # Skip creation of translations for choices in selects. The creation of these
+            # translations is done futher below in this function.
             parent = element.get("parent")
-            if parent and not parent.get("choice_filter"):
+            if parent is not None and parent[constants.TYPE] not in select_types:
                 for d in element.get_translations(self.default_language):
                     translation_path = d["path"]
                     form = "long"
@@ -665,37 +761,15 @@ class Survey(Section):
                             form: {
                                 "text": d["text"],
                                 "output_context": d["output_context"],
-                            }
+                            },
+                            constants.TYPE: constants.QUESTION,
                         }
                     )
 
-        # This code sets up translations for choices in filtered selects.
-        for list_name, choice_list in self.choices.items():
-            multi_language = isinstance(choice_list[0].get("label"), dict)
-            has_media = bool(choice_list[0].get("media"))
-            if (
-                not multi_language
-                and not has_media
-                and not has_dynamic_label(choice_list, multi_language)
-            ):
-                continue
-            for idx, choice in zip(range(len(choice_list)), choice_list):
-                for name, choice_value in choice.items():
-                    itext_id = "-".join([list_name, str(idx)])
-                    if isinstance(choice_value, dict):
-                        _setup_choice_translations(name, choice_value, itext_id)
-                    elif name == "label":
-                        self._add_to_nested_dict(
-                            self._translations,
-                            [self.default_language, itext_id, "long"],
-                            choice_value,
-                        )
-
     def _add_empty_translations(self):
         """
-        Adds translations so that every itext element has the same elements \
-        accross every language.
-        When translations are not provided "-" will be used.
+        Adds translations so that every itext element has the same elements across every
+        language. When translations are not provided "-" will be used.
         This disables any of the default_language fallback functionality.
         """
         paths = {}
@@ -747,7 +821,6 @@ class Survey(Section):
 
                     # Create the required dictionaries in _translations,
                     # then add media as a leaf value:
-
                     if language not in self._translations:
                         self._translations[language] = {}
 
@@ -763,18 +836,14 @@ class Survey(Section):
 
                     translations_trans_key[media_type] = media
 
-        if not self._translations:
-            self._translations = defaultdict(dict)  # pylint: disable=W0201
-
         for survey_element in self.iter_descendants():
-            # Skip set up of media for choices in filtered selects.
-            # Translations for the media content should have been set up
-            # in _setup_translations
-            parent = survey_element.get("parent")
-            if parent and not parent.get("choice_filter"):
-                translation_key = survey_element.get_xpath() + ":label"
+            # Skip set up of media for choices in selects. Translations for their media
+            # content should have been set up in _setup_translations, with one copy of
+            # each choice translation per language (after _add_empty_translations).
+            if not isinstance(survey_element, Option):
                 media_dict = survey_element.get("media")
-                if isinstance(media_dict, dict):
+                if isinstance(media_dict, dict) and 0 < len(media_dict):
+                    translation_key = survey_element.get_xpath() + ":label"
                     _set_up_media_translations(media_dict, translation_key)
 
     def itext(self):
@@ -799,6 +868,9 @@ class Survey(Section):
                     raise Exception()
 
                 for media_type, media_value in content.items():
+                    # Ignore key indicating Question or Choice translation type.
+                    if media_type == constants.TYPE:
+                        continue
                     if isinstance(media_value, dict):
                         value, output_inserted = self.insert_output_values(
                             media_value["text"], context=media_value["output_context"]
@@ -870,10 +942,10 @@ class Survey(Section):
         # TODO: check out pyxml
         # http://ronrothman.com/public/leftbraned/xml-dom-minidom-toprettyxml-and-silly-whitespace/
         xml_with_linebreaks = self.xml().toprettyxml(indent="  ")
-        text_re = re.compile(r"(>)\n\s*(\s[^<>\s].*?)\n\s*(\s</)", re.DOTALL)
-        output_re = re.compile(r"\n.*(<output.*>)\n(\s\s)*")
-        pretty_xml = text_re.sub(lambda m: "".join(m.group(1, 2, 3)), xml_with_linebreaks)
-        inline_output = output_re.sub(r"\g<1>", pretty_xml)
+        pretty_xml = RE_XML_TEXT.sub(
+            lambda m: "".join(m.group(1, 2, 3)), xml_with_linebreaks
+        )
+        inline_output = RE_XML_OUTPUT.sub(r"\g<1>", pretty_xml)
         return '<?xml version="1.0"?>\n' + inline_output
 
     def __repr__(self):
@@ -902,19 +974,15 @@ class Survey(Section):
         name = matchobj.group(2)
         last_saved = matchobj.group(1) is not None
         is_indexed_repeat = matchobj.string.find("indexed-repeat(") > -1
-        indexed_repeat_regex = re.compile(r"indexed-repeat\([^)]+\)")
-        function_args_regex = re.compile(r"\b[^()]+\((.*)\)$")
-        instance_regex = re.compile(r"instance\([^)]+.+")
-        bracket_regex = re.compile(r"\[([^]]+)\]")
 
-        def _in_secondary_instance_predicate():
+        def _in_secondary_instance_predicate() -> bool:
             """
             check if ${} expression represented by matchobj
             is in a predicate for a path expression for a secondary instance
             """
 
-            if instance_regex.search(matchobj.string) is not None:
-                bracket_regex_match_iter = bracket_regex.finditer(matchobj.string)
+            if RE_INSTANCE.search(matchobj.string) is not None:
+                bracket_regex_match_iter = RE_BRACKET.finditer(matchobj.string)
                 # Check whether current ${varname} is in the correct bracket_regex_match
                 for bracket_regex_match in bracket_regex_match_iter:
                     if (
@@ -925,10 +993,11 @@ class Survey(Section):
                 return False
             return False
 
-        def _relative_path(name):
+        def _relative_path(ref_name: str, _use_current: bool) -> Optional[str]:
             """Given name in ${name}, return relative xpath to ${name}."""
             return_path = None
-            xpath, context_xpath = self._xpath[name], context.get_xpath()
+            xpath = self._xpath[ref_name]
+            context_xpath = context.get_xpath()
             # share same root i.e repeat_a from /data/repeat_a/...
             if (
                 len(context_xpath.split("/")) > 2
@@ -940,14 +1009,13 @@ class Survey(Section):
                     self, xpath, context_xpath, reference_parent
                 )
                 if steps:
-                    ref_path = ref_path if ref_path.endswith(name) else "/%s" % name
-                    prefix = " current()/" if use_current else " "
-
+                    ref_path = ref_path if ref_path.endswith(ref_name) else "/%s" % name
+                    prefix = " current()/" if _use_current else " "
                     return_path = prefix + "/".join([".."] * steps) + ref_path + " "
 
             return return_path
 
-        def _is_return_relative_path():
+        def _is_return_relative_path() -> bool:
             """Determine condition to return relative xpath of current ${name}."""
             indexed_repeat_relative_path_args_index = [0, 1, 3, 5]
             current_matchobj = matchobj
@@ -958,7 +1026,7 @@ class Survey(Section):
                     return True
 
                 # It is possible to have multiple indexed-repeat in an expression
-                indexed_repeats_iter = indexed_repeat_regex.finditer(matchobj.string)
+                indexed_repeats_iter = RE_INDEXED_REPEAT.finditer(matchobj.string)
                 for indexed_repeat in indexed_repeats_iter:
 
                     # Make sure current ${name} is in the correct indexed-repeat
@@ -978,7 +1046,7 @@ class Survey(Section):
 
                     indexed_repeat_name_index = None
                     indexed_repeat_args = (
-                        function_args_regex.match(indexed_repeat.group())
+                        RE_FUNCTION_ARGS.search(indexed_repeat.group())
                         .group(1)
                         .split(",")
                     )
@@ -1009,7 +1077,7 @@ class Survey(Section):
         if _is_return_relative_path():
             if not use_current:
                 use_current = _in_secondary_instance_predicate()
-            relative_path = _relative_path(name)
+            relative_path = _relative_path(ref_name=name, _use_current=use_current)
             if relative_path:
                 return relative_path
 
@@ -1037,11 +1105,19 @@ class Survey(Section):
         """
         return '<output value="' + self._var_repl_function(matchobj, context) + '" />'
 
-    def insert_output_values(self, text, context=None):
+    def insert_output_values(
+        self,
+        text: str,
+        context: Optional[SurveyElement] = None,
+    ) -> Tuple[str, bool]:
         """
         Replace all the ${variables} in text with xpaths.
         Returns that and a boolean indicating if there were any ${variables}
         present.
+
+        :param text: Input text to process.
+        :param context: The document node that the text belongs to.
+        :return: The output text, and a flag indicating whether any changes were made.
         """
 
         def _var_repl_output_function(matchobj):
@@ -1054,15 +1130,19 @@ class Survey(Section):
         # variable replacement:
         text_node = PatchedText()
         text_node.data = text
-        xml_text = text_node.toxml()
+        original_xml = text_node.toxml()
 
         # need to make sure we have reason to replace
         # since at this point < is &lt,
         # the net effect &lt gets translated again to &amp;lt;
-        if str(xml_text).find("{") != -1:
-            result = re.sub(BRACKETED_TAG_REGEX, _var_repl_output_function, str(xml_text))
-            return result, not result == xml_text
-        return text, False
+        xml_text = instance_expression.replace_with_output(original_xml, context, self)
+        if "{" in xml_text:
+            xml_text = re.sub(BRACKETED_TAG_REGEX, _var_repl_output_function, xml_text)
+        changed = xml_text != original_xml
+        if changed:
+            return xml_text, True
+        else:
+            return text, False
 
     # pylint: disable=too-many-arguments
     def print_xform_to_file(
