@@ -4,6 +4,7 @@ Survey Element base class for all survey elements.
 """
 import json
 import re
+from collections import deque
 from functools import lru_cache
 from typing import TYPE_CHECKING
 
@@ -188,16 +189,14 @@ class SurveyElement(dict):
         """
         Return a the list [root, ..., self._parent, self]
         """
-        result = [self]
+        result = deque((self,))
         current_element = self
         while current_element.parent:
             current_element = current_element.parent
-            result = [current_element] + result
+            result.appendleft(current_element)
         # For some reason the root element has a True flat property...
-        output = [result[0]]
-        for item in result[1:]:
-            if not item.get("flat"):
-                output.append(item)
+        output = [result.popleft()]
+        output.extend([i for i in result if not i.get("flat")])
         return output
 
     def get_root(self):
@@ -244,6 +243,13 @@ class SurveyElement(dict):
         result["children"] = []
         for child in children:
             result["children"].append(child.to_json_dict())
+        # Translation items with "output_context" have circular references.
+        if "_translations" in result:
+            for lang in result["_translations"].values():
+                for item in lang.values():
+                    for form in item.values():
+                        if callable(getattr(form, "pop", None)):
+                            form.pop("output_context", None)
         # remove any keys with empty values
         for k, v in list(result.items()):
             if not v:
@@ -334,7 +340,7 @@ class SurveyElement(dict):
             # how they're defined - https://opendatakit.github.io/xforms-spec/#languages
             if (
                 display_element == "guidance_hint"
-                and not (isinstance(label_or_hint, dict))
+                and not isinstance(label_or_hint, dict)
                 and len(label_or_hint) > 0
             ):
                 label_or_hint = {default_language: label_or_hint}
@@ -342,7 +348,7 @@ class SurveyElement(dict):
             # always use itext for hint if there's a guidance hint
             if (
                 display_element == "hint"
-                and not (isinstance(label_or_hint, dict))
+                and not isinstance(label_or_hint, dict)
                 and len(label_or_hint) > 0
                 and "guidance_hint" in self.keys()
                 and len(self["guidance_hint"]) > 0
@@ -425,7 +431,7 @@ class SurveyElement(dict):
                 result.append(self.xml_label())
             result.append(self.xml_hint())
 
-        msg = "The survey element named '%s' " "has no label or hint." % self.name
+        msg = "The survey element named '%s' has no label or hint." % self.name
         if len(result) == 0:
             raise PyXFormError(msg)
 
