@@ -10,7 +10,7 @@ import os
 import re
 from collections import namedtuple
 from json.decoder import JSONDecodeError
-from typing import List, Tuple
+from typing import Dict, List, Tuple
 from xml.dom.minidom import Element, Text, parseString
 
 import openpyxl
@@ -25,6 +25,8 @@ INVALID_XFORM_TAG_REGEXP = r"[^a-zA-Z:_][^a-zA-Z:_0-9\-.]*"
 LAST_SAVED_INSTANCE_NAME = "__last-saved"
 BRACKETED_TAG_REGEX = re.compile(r"\${(last-saved#)?(.*?)}")
 LAST_SAVED_REGEX = re.compile(r"\${last-saved#(.*?)}")
+PYXFORM_REFERENCE_REGEX = re.compile(r"\$\{(.*?)\}")
+
 
 NSMAP = {
     "xmlns": "http://www.w3.org/2002/xforms",
@@ -62,7 +64,7 @@ class PatchedText(Text):
         writer.write(data)
 
 
-def node(*args, **kwargs):
+def node(*args, **kwargs) -> DetachableElement:
     """
     args[0] -- a XML tag
     args[1:] -- an array of children to append to the newly created node
@@ -78,10 +80,8 @@ def node(*args, **kwargs):
     assert len(unicode_args) <= 1
     parsed_string = False
 
-    # Convert the kwargs xml attribute dictionary to a xml.dom.minidom.Element. Sort the
-    # attributes to guarantee a consistent order across Python versions.
-    # See pyxform_test_case.reorder_attributes for details.
-    for k, v in iter(sorted(kwargs.items())):
+    # Convert the kwargs xml attribute dictionary to a xml.dom.minidom.Element.
+    for k, v in iter(kwargs.items()):
         if k in blocked_attributes:
             continue
         if k == "toParseString":
@@ -182,7 +182,7 @@ def xls_sheet_to_csv(workbook_path, csv_path, sheet_name):
 def xlsx_sheet_to_csv(workbook_path, csv_path, sheet_name):
     wb = openpyxl.open(workbook_path, read_only=True, data_only=True)
     try:
-        sheet = wb.get_sheet_by_name(sheet_name)
+        sheet = wb[sheet_name]
     except KeyError:
         return False
 
@@ -236,7 +236,7 @@ def get_languages_with_bad_tags(languages):
         lang_code = re.search(lang_code_regex, lang)
 
         if lang != "default" and (
-            not (lang_code) or not (lang_code.group(1) in iana_subtags)
+            not lang_code or not lang_code.group(1) in iana_subtags
         ):
             languages_with_bad_tags.append(lang)
     return languages_with_bad_tags
@@ -275,17 +275,20 @@ def default_is_dynamic(element_default, element_type=None):
     return False
 
 
-# If the first or second choice label includes a reference, we must use itext.
-# Check the first two choices in case first is something like "Other".
-def has_dynamic_label(choice_list, multi_language):
-    if not multi_language:
-        for i in range(0, min(2, len(choice_list))):
-            if (
-                choice_list[i].get("label") is not None
-                and re.search(BRACKETED_TAG_REGEX, choice_list[i].get("label"))
-                is not None
-            ):
-                return True
+def has_dynamic_label(choice_list: "List[Dict[str, str]]") -> bool:
+    """
+    If the first or second choice label includes a reference, we must use itext.
+
+    Check the first two choices in case first is something like "Other".
+    """
+    for c in choice_list[:2]:
+        choice_label = c.get("label")
+        if (
+            choice_label is not None
+            and isinstance(choice_label, str)
+            and re.search(BRACKETED_TAG_REGEX, choice_label) is not None
+        ):
+            return True
     return False
 
 
@@ -385,7 +388,8 @@ def get_expression_lexer() -> re.Scanner:  # noqa
         "WHITESPACE": r"\s+",
         "PYXFORM_REF": r"\$\{" + ncname_regex + r"(#" + ncname_regex + r")?" + r"\}",
         "FUNC_CALL": ncname_regex + r"\(",
-        "XPATH_PRED": ncname_regex + r"\[",
+        "XPATH_PRED_START": ncname_regex + r"\[",
+        "XPATH_PRED_END": r"\]",
         "URI_SCHEME": ncname_regex + r"://",
         "NAME": ncname_regex,  # Must be after rules containing ncname_regex.
         "OTHER": r".+?",  # Catch any other character so that parsing doesn't stop.
