@@ -4,6 +4,9 @@ from pyxform.errors import PyXFormError
 
 PARAMETERS_TYPE = Dict[str, Any]
 
+# Label and value are used to match against user-specified files so case should be preserved.
+CASE_SENSITIVE_VALUES = ["label", "value"]
+
 
 def parse(raw_parameters: str) -> PARAMETERS_TYPE:
     parts = raw_parameters.split(";")
@@ -21,7 +24,7 @@ def parse(raw_parameters: str) -> PARAMETERS_TYPE:
             )
         k, v = param.split("=")[:2]
         key = k.lower().strip()
-        params[key] = v.lower().strip()
+        params[key] = v.strip() if key in CASE_SENSITIVE_VALUES else v.lower().strip()
 
     return params
 
