@@ -8,7 +8,7 @@ import os
 import re
 import sys
 from collections import Counter
-from typing import Any, Dict, List
+from typing import IO, Any, Dict, List, Optional, Tuple
 
 from pyxform import aliases, constants
 from pyxform.constants import (
@@ -22,15 +22,16 @@ from pyxform.entities.entities_parsing import (
     validate_entity_saveto,
 )
 from pyxform.errors import PyXFormError
-from pyxform.utils import default_is_dynamic
-from pyxform.validators.pyxform import parameters_generic, select_from_file_params
-from pyxform.validators.pyxform.missing_translations_check import (
-    missing_translations_check,
-)
+from pyxform.parsing.expression import is_single_token_expression
+from pyxform.utils import PYXFORM_REFERENCE_REGEX, default_is_dynamic
+from pyxform.validators.pyxform import parameters_generic, select_from_file
+from pyxform.validators.pyxform.android_package_name import validate_android_package_name
+from pyxform.validators.pyxform.translations_checks import SheetTranslations
 from pyxform.xls2json_backends import csv_to_dict, xls_to_dict, xlsx_to_dict
 from pyxform.xlsparseutils import find_sheet_misspellings, is_valid_xml_tag
 
 SMART_QUOTES = {"\u2018": "'", "\u2019": "'", "\u201c": '"', "\u201d": '"'}
+SEARCH_APPEARANCE_REGEX = re.compile(r"search\(.*?\)")
 
 
 def print_pyobj_to_json(pyobj, path=None):
@@ -58,19 +59,22 @@ def merge_dicts(dict_a, dict_b, default_key="default"):
     if dict_b is None or dict_b == {}:
         return dict_a
 
-    if type(dict_a) is not dict:
+    if not isinstance(dict_a, dict):
         if default_key in dict_b:
             return dict_b
         dict_a = {default_key: dict_a}
-    if type(dict_b) is not dict:
+    if not isinstance(dict_b, dict):
         if default_key in dict_a:
             return dict_a
         dict_b = {default_key: dict_b}
 
-    all_keys = set(dict_a.keys()).union(set(dict_b.keys()))
+    # Union keys but retain order (as opposed to set()), preferencing dict_a then dict_b.
+    # E.g. {"a": 1, "b": 2} + {"c": 3, "a": 4} -> {"a": None, "b": None, "c": None}
+    all_keys = {k: None for k in dict_a.keys()}
+    all_keys.update({k: None for k in dict_b.keys()})
 
     out_dict = dict()
-    for key in all_keys:
+    for key in all_keys.keys():
         out_dict[key] = merge_dicts(dict_a.get(key), dict_b.get(key), default_key)
     return out_dict
 
@@ -97,13 +101,25 @@ def replace_smart_quotes_in_dict(_d):
             _d[key] = value
 
 
+class DealiasAndGroupHeadersResult:
+    __slots__ = ("headers", "data")
+
+    def __init__(self, headers: Tuple[Tuple[str, ...], ...], data: List[Dict]):
+        """
+        :param headers: Distinct headers seen in the sheet, parsed / split if applicable.
+        :param data: Sheet data rows, in grouped dict format.
+        """
+        self.headers: Tuple[Tuple[str, ...], ...] = headers
+        self.data: List[Dict] = data
+
+
 def dealias_and_group_headers(
-    dict_array: "List[Dict]",
-    header_aliases: "Dict",
+    dict_array: List[Dict],
+    header_aliases: Dict[str, str],
     use_double_colons: bool,
     default_language: str = constants.DEFAULT_LANGUAGE_VALUE,
     ignore_case: bool = False,
-):
+) -> DealiasAndGroupHeadersResult:
     """
     For each row in the worksheet, group all keys that contain a double colon.
     So
@@ -117,10 +133,10 @@ def dealias_and_group_headers(
     """
     group_delimiter = "::"
     out_dict_array = list()
+    seen_headers = {}
     for row in dict_array:
         out_row = dict()
         for header, val in row.items():
-
             if ignore_case:
                 header = header.lower()
 
@@ -150,9 +166,12 @@ def dealias_and_group_headers(
             new_key = tokens[0]
             new_value = list_to_nested_dict(tokens[1:] + [val])
             out_row = merge_dicts(out_row, {new_key: new_value}, default_language)
+            seen_headers[tuple(tokens)] = None
 
         out_dict_array.append(out_row)
-    return out_dict_array
+    return DealiasAndGroupHeadersResult(
+        headers=tuple(seen_headers.keys()), data=out_dict_array
+    )
 
 
 def dealias_types(dict_array):
@@ -322,13 +341,74 @@ def process_image_default(default_value):
     return default_value
 
 
+def add_choices_info_to_question(
+    question: Dict[str, Any],
+    list_name: str,
+    choices: Dict[str, list],
+    choice_filter: str,
+    file_extension: str,
+):
+    """
+    Add choices-related info to the question dict, e.g. itemset, list_name, choices, etc.
+
+    :param question: A dict with question details.
+    :param list_name: The choice list name for the question.
+    :param choices: The available choices in the survey.
+    :param choice_filter: The question's choice_filter, if any.
+    :param file_extension: The question's external select file_extension, if any.
+    :return: The updated question dict.
+    """
+    if choice_filter is None:
+        choice_filter = ""
+    if file_extension is None:
+        file_extension = ""
+    try:
+        is_search = bool(
+            SEARCH_APPEARANCE_REGEX.search(
+                question[constants.CONTROL][constants.APPEARANCE]
+            )
+        )
+    except (KeyError, TypeError):
+        is_search = False
+
+    # External selects from a "search" appearance alone don't work in Enketo. In Collect
+    # they must have the "item" elements in the body, rather than in an "itemset".
+    if not is_search:
+        question[constants.ITEMSET] = list_name
+
+    if choice_filter:
+        # External selects e.g. type = "select_one_external city".
+        if question[constants.TYPE] == constants.SELECT_ONE_EXTERNAL:
+            question["query"] = list_name
+        elif choices.get(list_name):
+            # Reference to list name for data dictionary tools (ilri/odktools).
+            question["list_name"] = list_name
+            # Copy choices for data export tools (onaio/onadata).
+            # TODO: could onadata use the list_name to look up the list for
+            #  export, instead of pyxform internally duplicating choices data?
+            question[constants.CHOICES] = choices[list_name]
+    elif not (
+        # Select with randomized choices.
+        (
+            constants.RANDOMIZE in question[constants.PARAMETERS]
+            and question[constants.PARAMETERS][constants.RANDOMIZE] == "true"
+        )
+        # Select from file e.g. type = "select_one_from_file cities.xml".
+        or file_extension in EXTERNAL_INSTANCE_EXTENSIONS
+        # Select from previous answers e.g. type = "select_one ${q1}".
+        or bool(PYXFORM_REFERENCE_REGEX.search(list_name))
+    ):
+        question["list_name"] = list_name
+        question[constants.CHOICES] = choices[list_name]
+
+
 def workbook_to_json(
     workbook_dict,
-    form_name=None,
-    fallback_form_name=None,
-    default_language=constants.DEFAULT_LANGUAGE_VALUE,
-    warnings=None,
-) -> "Dict[str, Any]":
+    form_name: Optional[str] = None,
+    fallback_form_name: Optional[str] = None,
+    default_language: str = constants.DEFAULT_LANGUAGE_VALUE,
+    warnings: Optional[List[str]] = None,
+) -> Dict[str, Any]:
     """
     workbook_dict -- nested dictionaries representing a spreadsheet.
                     should be similar to those returned by xls_to_dict
@@ -413,9 +493,11 @@ def workbook_to_json(
         settings_sheet_headers = []
 
     settings_sheet = dealias_and_group_headers(
-        settings_sheet_headers, aliases.settings_header, use_double_colons
+        dict_array=settings_sheet_headers,
+        header_aliases=aliases.settings_header,
+        use_double_colons=use_double_colons,
     )
-    settings = settings_sheet[0] if len(settings_sheet) > 0 else {}
+    settings = settings_sheet.data[0] if len(settings_sheet.data) > 0 else {}
     replace_smart_quotes_in_dict(settings)
 
     default_language = settings.get(constants.DEFAULT_LANGUAGE_KEY, default_language)
@@ -453,10 +535,13 @@ def workbook_to_json(
         replace_smart_quotes_in_dict(choice_item)
 
     external_choices_sheet = dealias_and_group_headers(
-        external_choices_sheet, aliases.list_header, use_double_colons, default_language
+        dict_array=external_choices_sheet,
+        header_aliases=aliases.list_header,
+        use_double_colons=use_double_colons,
+        default_language=default_language,
     )
     external_choices = group_dictionaries_by_key(
-        external_choices_sheet, constants.LIST_NAME
+        list_of_dicts=external_choices_sheet.data, key=constants.LIST_NAME
     )
 
     # ########## Choices sheet ##########
@@ -464,9 +549,14 @@ def workbook_to_json(
     for choice_item in choices_sheet:
         replace_smart_quotes_in_dict(choice_item)
     choices_sheet = dealias_and_group_headers(
-        choices_sheet, aliases.list_header, use_double_colons, default_language
+        dict_array=choices_sheet,
+        header_aliases=aliases.list_header,
+        use_double_colons=use_double_colons,
+        default_language=default_language,
+    )
+    combined_lists = group_dictionaries_by_key(
+        list_of_dicts=choices_sheet.data, key=constants.LIST_NAME
     )
-    combined_lists = group_dictionaries_by_key(choices_sheet, constants.LIST_NAME)
     # To combine the warning into one message, the check for missing choices translation
     # columns is run with Survey sheet below.
 
@@ -538,7 +628,15 @@ def workbook_to_json(
                             )  # noqa
 
     # ########## Entities sheet ###########
-    entity_declaration = get_entity_declaration(workbook_dict, warnings)
+    entities_sheet = workbook_dict.get(constants.ENTITIES, [])
+    entities_sheet = dealias_and_group_headers(
+        dict_array=entities_sheet,
+        header_aliases=aliases.entities_header,
+        use_double_colons=False,
+    )
+    entity_declaration = get_entity_declaration(
+        entities_sheet=entities_sheet.data, workbook_dict=workbook_dict, warnings=warnings
+    )
 
     # ########## Survey sheet ###########
     survey_sheet = workbook_dict[constants.SURVEY]
@@ -549,23 +647,30 @@ def workbook_to_json(
     if clean_text_values_enabled:
         survey_sheet = clean_text_values(survey_sheet)
     survey_sheet = dealias_and_group_headers(
-        survey_sheet, aliases.survey_header, use_double_colons, default_language
+        dict_array=survey_sheet,
+        header_aliases=aliases.survey_header,
+        use_double_colons=use_double_colons,
+        default_language=default_language,
     )
-    survey_sheet = dealias_types(survey_sheet)
+    survey_sheet.data = dealias_types(dict_array=survey_sheet.data)
 
     # Check for missing translations. The choices sheet is checked here so that the
     # warning can be combined into one message.
-    warnings = missing_translations_check(
-        survey_sheet=survey_sheet,
-        choices_sheet=choices_sheet,
-        warnings=warnings,
+    sheet_translations = SheetTranslations(
+        survey_sheet=survey_sheet.headers,
+        choices_sheet=choices_sheet.headers,
     )
+    sheet_translations.missing_check(warnings=warnings)
 
     # No spell check for OSM sheet (infrequently used, many spurious matches).
     osm_sheet = dealias_and_group_headers(
-        workbook_dict.get(constants.OSM, []), aliases.list_header, True
+        dict_array=workbook_dict.get(constants.OSM, []),
+        header_aliases=aliases.list_header,
+        use_double_colons=True,
+    )
+    osm_tags = group_dictionaries_by_key(
+        list_of_dicts=osm_sheet.data, key=constants.LIST_NAME
     )
-    osm_tags = group_dictionaries_by_key(osm_sheet, constants.LIST_NAME)
     # #################################
 
     # Parse the survey sheet while generating a survey in our json format:
@@ -608,7 +713,7 @@ def workbook_to_json(
 
     # row by row, validate questions, throwing errors and adding warnings
     # where needed.
-    for row in survey_sheet:
+    for row in survey_sheet.data:
         row_number += 1
         if stack[-1] is not None:
             prev_control_type = stack[-1]["control_type"]
@@ -639,7 +744,7 @@ def workbook_to_json(
         if not question_type:
             # if name and label are also missing,
             # then its a comment row, and we skip it with warning
-            if not ((constants.NAME in row) or (constants.LABEL in row)):
+            if not (constants.NAME in row or constants.LABEL in row):
                 warnings.append(
                     ROW_FORMAT_STRING % row_number
                     + " Row without name, text, or label is being skipped:\n"
@@ -952,24 +1057,29 @@ def workbook_to_json(
                         )
                     new_json_dict[constants.COLUMNS] = choices[list_name]
 
-                # Generate a new node for the jr:count column so
-                # xpath expressions can be used.
+                # Generate a new node for the jr:count column so xpath expressions can be used.
                 repeat_count_expression = new_json_dict.get("control", {}).get("jr:count")
                 if repeat_count_expression:
-                    generated_node_name = new_json_dict["name"] + "_count"
-                    parent_children_array.append(
-                        {
-                            "name": generated_node_name,
-                            "bind": {
-                                "readonly": "true()",
-                                "calculate": repeat_count_expression,
-                            },
-                            "type": "calculate",
-                        }
-                    )
-                    new_json_dict["control"]["jr:count"] = (
-                        "${" + generated_node_name + "}"
+                    # Simple expressions don't require a new node, they can reference directly.
+                    simple_expression = is_single_token_expression(
+                        expression=repeat_count_expression, token_types=["PYXFORM_REF"]
                     )
+                    if not simple_expression:
+                        generated_node_name = new_json_dict["name"] + "_count"
+                        parent_children_array.append(
+                            {
+                                "name": generated_node_name,
+                                "bind": {
+                                    "readonly": "true()",
+                                    "calculate": repeat_count_expression,
+                                },
+                                "type": "calculate",
+                            }
+                        )
+                        # This re-directs the body/repeat ref to the above generated node.
+                        new_json_dict["control"]["jr:count"] = (
+                            "${" + generated_node_name + "}"
+                        )
 
                 # Code to deal with table_list appearance flags
                 # (for groups of selects)
@@ -1024,16 +1134,19 @@ def workbook_to_json(
             parse_dict = select_parse.groupdict()
             if parse_dict.get("select_command"):
                 select_type = aliases.select[parse_dict["select_command"]]
-                if select_type == "select one external" and "choice_filter" not in row:
+                if (
+                    select_type == constants.SELECT_ONE_EXTERNAL
+                    and constants.CHOICE_FILTER not in row
+                ):
                     warnings.append(
                         ROW_FORMAT_STRING % row_number
                         + " select one external is only meant for"
                         " filtered selects."
                     )
                 list_name = parse_dict["list_name"]
-                list_file_name, file_extension = os.path.splitext(list_name)
+                file_extension = os.path.splitext(list_name)[1]
                 if (
-                    select_type == "select one external"
+                    select_type == constants.SELECT_ONE_EXTERNAL
                     and list_name not in external_choices
                 ):
                     if not external_choices:
@@ -1052,11 +1165,16 @@ def workbook_to_json(
                         + "List name not in external choices sheet: "
                         + list_name
                     )
+                select_from_file.validate_list_name_extension(
+                    select_command=parse_dict["select_command"],
+                    list_name=list_name,
+                    row_number=row_number,
+                )
                 if (
                     list_name not in choices
-                    and select_type != "select one external"
+                    and select_type != constants.SELECT_ONE_EXTERNAL
                     and file_extension not in EXTERNAL_INSTANCE_EXTENSIONS
-                    and not re.match(r"\$\{(.*?)\}", list_name)
+                    and not PYXFORM_REFERENCE_REGEX.search(list_name)
                 ):
                     if not choices:
                         k = constants.CHOICES
@@ -1093,7 +1211,14 @@ def workbook_to_json(
 
                 specify_other_question = None
                 if parse_dict.get("specify_other") is not None:
-                    select_type += " or specify other"
+                    sheet_translations.or_other_seen = True
+                    select_type += constants.SELECT_OR_OTHER_SUFFIX
+                    if row.get(constants.CHOICE_FILTER):
+                        msg = (
+                            ROW_FORMAT_STRING % row_number
+                            + " Choice filter not supported with or_other."
+                        )
+                        raise PyXFormError(msg)
 
                 new_json_dict = row.copy()
                 new_json_dict[constants.TYPE] = select_type
@@ -1134,41 +1259,37 @@ def workbook_to_json(
                     )
 
                 if "value" in parameters.keys():
-                    select_from_file_params.value_or_label_check(
+                    select_from_file.value_or_label_check(
                         name="value",
                         value=parameters["value"],
                         row_number=row_number,
                     )
                 if "label" in parameters.keys():
-                    select_from_file_params.value_or_label_check(
+                    select_from_file.value_or_label_check(
                         name="label",
                         value=parameters["label"],
                         row_number=row_number,
                     )
 
-                new_json_dict["parameters"] = parameters
+                new_json_dict[constants.PARAMETERS] = parameters
 
-                if row.get("choice_filter"):
-                    if select_type == "select one external":
-                        new_json_dict["query"] = list_name
-                    else:
-                        new_json_dict["itemset"] = list_name
-                        json_dict["choices"] = choices
-                        if choices.get(list_name):
-                            new_json_dict["list_name"] = list_name
-                            new_json_dict[constants.CHOICES] = choices[list_name]
-                elif (
-                    "randomize" in parameters.keys() and parameters["randomize"] == "true"
+                add_choices_info_to_question(
+                    question=new_json_dict,
+                    list_name=list_name,
+                    choices=choices,
+                    choice_filter=row.get(constants.CHOICE_FILTER),
+                    file_extension=file_extension,
+                )
+                # Add the choice to the survey choices if it's being used in an itemset.
+                if (
+                    constants.ITEMSET in new_json_dict
+                    and new_json_dict[constants.ITEMSET] == list_name
+                    and list_name in choices
                 ):
-                    new_json_dict["itemset"] = list_name
-                    json_dict["choices"] = choices
-                elif file_extension in EXTERNAL_INSTANCE_EXTENSIONS or re.match(
-                    r"\$\{(.*?)\}", list_name
-                ):
-                    new_json_dict["itemset"] = list_name
-                else:
-                    new_json_dict["list_name"] = list_name
-                    new_json_dict[constants.CHOICES] = choices[list_name]
+                    # Initialise choices output if none added already.
+                    if constants.CHOICES not in json_dict:
+                        json_dict[constants.CHOICES] = {}
+                    json_dict[constants.CHOICES][list_name] = choices[list_name]
 
                 # Code to deal with table_list appearance flags
                 # (for groups of selects)
@@ -1176,15 +1297,20 @@ def workbook_to_json(
                     # Then this row is the first select in a table list
                     if not isinstance(table_list, str):
                         table_list = list_name
+                        if row.get(constants.CHOICE_FILTER, None) is not None:
+                            msg = (
+                                ROW_FORMAT_STRING % row_number
+                                + " Choice filter not supported for table-list appearance."
+                            )
+                            raise PyXFormError(msg)
                         table_list_header = {
                             constants.TYPE: select_type,
                             constants.NAME: "reserved_name_for_field_list_labels_"
                             + str(row_number),
                             # Adding row number for uniqueness # noqa
-                            constants.CONTROL: {"appearance": "label"},
+                            constants.CONTROL: {constants.APPEARANCE: "label"},
                             constants.CHOICES: choices[list_name],
-                            # Do we care about filtered selects in table lists?
-                            # 'itemset' : list_name,
+                            constants.ITEMSET: list_name,
                         }
                         parent_children_array.append(table_list_header)
 
@@ -1196,8 +1322,11 @@ def workbook_to_json(
                         )
                         raise PyXFormError(error_message)
 
-                    control = new_json_dict["control"] = new_json_dict.get("control", {})
-                    control["appearance"] = "list-nolabel"
+                    if constants.CONTROL not in new_json_dict:
+                        new_json_dict[constants.CONTROL] = {}
+                    new_json_dict[constants.CONTROL][
+                        constants.APPEARANCE
+                    ] = constants.LIST_NOLABEL
                 parent_children_array.append(new_json_dict)
                 if specify_other_question:
                     parent_children_array.append(specify_other_question)
@@ -1227,12 +1356,37 @@ def workbook_to_json(
             parent_children_array.append(new_dict)
             continue
 
+        if question_type == "text":
+            new_dict = row.copy()
+            parameters_generic.validate(parameters=parameters, allowed=("rows",))
+
+            if "rows" in parameters.keys():
+                try:
+                    int(parameters["rows"])
+                except ValueError:
+                    raise PyXFormError(
+                        (ROW_FORMAT_STRING % row_number)
+                        + " Parameter rows must have an integer value."
+                    )
+
+                new_dict["control"] = new_dict.get("control", {})
+                new_dict["control"].update({"rows": parameters["rows"]})
+
+            parent_children_array.append(new_dict)
+            continue
+
         if question_type == "photo":
             new_dict = row.copy()
 
             if row.get("default"):
                 new_dict["default"] = process_image_default(row["default"])
-            parameters_generic.validate(parameters=parameters, allowed=("max-pixels",))
+            parameters_generic.validate(
+                parameters=parameters,
+                allowed=(
+                    "max-pixels",
+                    "app",
+                ),
+            )
             if "max-pixels" in parameters.keys():
                 try:
                     int(parameters["max-pixels"])
@@ -1246,6 +1400,20 @@ def workbook_to_json(
                     (ROW_FORMAT_STRING % row_number)
                     + " Use the max-pixels parameter to speed up submission sending and save storage space. Learn more: https://xlsform.org/#image"
                 )
+
+            if "app" in parameters.keys():
+                appearance = row.get("control", {}).get("appearance")
+                if appearance is None or appearance == "annotate":
+                    app_package_name = str(parameters["app"])
+                    validation_result = validate_android_package_name(app_package_name)
+                    if validation_result is None:
+                        new_dict["control"] = new_dict.get("control", {})
+                        new_dict["control"].update({"intent": app_package_name})
+                    else:
+                        raise PyXFormError(
+                            (ROW_FORMAT_STRING % row_number) + " " + validation_result
+                        )
+
             parent_children_array.append(new_dict)
             continue
 
@@ -1342,6 +1510,8 @@ def workbook_to_json(
         # Put the row in the json dict as is:
         parent_children_array.append(row)
 
+    sheet_translations.or_other_check(warnings=warnings)
+
     if len(stack) != 1:
         raise PyXFormError(
             "Unmatched begin statement: "
@@ -1381,7 +1551,11 @@ def workbook_to_json(
         )
 
     if len(entity_declaration) > 0:
-        json_dict[constants.ENTITY_RELATED] = "true"
+        json_dict[constants.ENTITY_FEATURES] = ["create"]
+
+        if entity_declaration.get("parameters", {}).get("entity_id", None):
+            json_dict[constants.ENTITY_FEATURES].append("update")
+
         meta_children.append(entity_declaration)
 
     if len(meta_children) > 0:
@@ -1431,12 +1605,12 @@ def get_filename(path):
 
 
 def parse_file_to_json(
-    path,
-    default_name="data",
-    default_language=constants.DEFAULT_LANGUAGE_VALUE,
-    warnings=None,
-    file_object=None,
-):
+    path: str,
+    default_name: str = "data",
+    default_language: str = constants.DEFAULT_LANGUAGE_VALUE,
+    warnings: Optional[List[str]] = None,
+    file_object: Optional[IO] = None,
+) -> Dict[str, Any]:
     """
     A wrapper for workbook_to_json
     """
@@ -1445,7 +1619,11 @@ def parse_file_to_json(
     workbook_dict = parse_file_to_workbook_dict(path, file_object)
     fallback_form_name = str(get_filename(path))
     return workbook_to_json(
-        workbook_dict, default_name, fallback_form_name, default_language, warnings
+        workbook_dict=workbook_dict,
+        form_name=default_name,
+        fallback_form_name=fallback_form_name,
+        default_language=default_language,
+        warnings=warnings,
     )
 
 
@@ -1541,7 +1719,7 @@ class QuestionTypesReader(SpreadsheetReader):
             header_aliases={},
             use_double_colons=use_double_colons,
             default_language=constants.DEFAULT_LANGUAGE_VALUE,
-        )
+        ).data
         self._dict = organize_by_values(self._dict, "name")
 
 
